/*
 * Commands.c
 *
 *  Created on: Oct 30, 2009
 *      Author: chwolfe2
 */

#include "Commands.h"
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/mman.h>
#include "Globals.h"
#include "MIB.h"
#include "HostInterface.h"

char packetbuf[8192];
char filename[8192];
MyFile tempfile;




StatusCode Command_Process(SafeMessage* message, SafeMessage* response){
	StatusCode sc;
	assert(message!=NULL && response!=NULL);
	char commandType[4];
	int i=0;for(i=0;i<3;i++)commandType[i]=message->type[i];commandType[3]='\0';
	if 		(strncmp("PNG",commandType,3)==0) return Command_PNG(message,response);
	else if (strncmp("RPT",commandType,3)==0) return Command_RPT(message,response);
	else if (strncmp("SHT",commandType,3)==0) return Command_SHT(message,response);
	else if (strncmp("INI",commandType,3)==0) return Command_INI(message,response);
	else if (strncmp("DEL",commandType,3)==0) return Command_DEL(message,response);
	else if (strncmp("STP",commandType,3)==0) return Command_STP(message,response);
	else if (strncmp("GET",commandType,3)==0) return Command_GET(message,response);
	else if (strncmp("DWN",commandType,3)==0) return Command_DWN(message,response);
	else if (strncmp("UP ",commandType,3)==0) return Command_UP(message,response);
	else if (strncmp(" UP",commandType,3)==0) return Command_UP(message,response);
	else if (strncmp("EJT",commandType,3)==0) return Command_EJT(message,response);
	else if (strncmp("REC",commandType,3)==0) return Command_REC(message,response);
	else if (strncmp("TST",commandType,3)==0) {
		sc=Command_TST(message,response);
		printf("Command_TST(message,response) = %s\n",statusString(sc));fflush(stdout);
		return sc;
		//return Command_TST(message,response);

	}
	else if (strncmp("CPY",commandType,3)==0) return Command_CPY(message,response);
	else if (strncmp("DMP",commandType,3)==0) return Command_DMP(message,response);
	else if (strncmp("FMT",commandType,3)==0) return Command_FMT(message,response);
	else if (strncmp("SYN",commandType,3)==0) return Command_SYN(message,response);
	else {
		return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_COMMAND, commandType);
	}
}

// Instant Commands
StatusCode Command_PNG(SafeMessage* message, SafeMessage* response){
	return Message_GenerateResponse(message, response, true, "", 0+1);
}
StatusCode Command_RPT(SafeMessage* message, SafeMessage* response){
	return  MIB_Get(message,response);
}

StatusCode Command_SHT(SafeMessage* message, SafeMessage* response){
	globals.exitRequested=true;
	globals.shutdownRequested=true;
	if (strstr(message->data.messageData,"SCRAM")!=NULL) globals.scramRequested=true;
	if (strstr(message->data.messageData,"REBOOT")!=NULL) globals.rebootRequested=true;
	return Message_GenerateResponse(message,response,true,"",0+1);
}

StatusCode Command_INI(SafeMessage* message, SafeMessage* response){
	globals.exitRequested=true;
	globals.shutdownRequested=false;
	globals.flushConfigRequested=true;
	globals.flushScheduleRequested=true;
	if (strstr(message->data.messageData,"L")!=NULL) globals.flushLogRequested = true;
	if (strstr(message->data.messageData,"D")!=NULL) globals.flushDataRequested = true;
	if (strstr(message->data.messageData,"--flush-log")!=NULL) globals.flushLogRequested = true;
	if (strstr(message->data.messageData,"--flush-data")!=NULL) globals.flushDataRequested = true;
	return Message_GenerateResponse(message,response,true,"",0+1);
}

StatusCode Command_DEL(SafeMessage* message, SafeMessage* response){

	return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
}

StatusCode Command_STP(SafeMessage* message, SafeMessage* response){
	char filename[20];
	size_t opReference;
	size_t mjd;
	int schedule_index=-1;
	StatusCode sc;
	strncpy(filename,message->data.messageData,20);
	filename[19]='\0';
	if (sscanf(filename,"%lu_%lu",&mjd,&opReference)!=2)
		return Message_GenerateResponse(message,response,false, "Bad format", strlen("Bad format")+1);

	opReference=strtoul(filename,NULL,10);
	if (globals.opData.opReference == opReference){
		Process_AbortCurrent();
		return Message_GenerateResponse(message, response, true, "", 0+1);
	} else {
		schedule_index = Schedule_getIndexByReference(opReference);
		if (schedule_index!=-1){
			sc=Schedule_RemoveEntry(opReference);
			if (sc!=SUCCESS){
				printf("Error: failed to remove item %d with reference %lu from schedule.\n",schedule_index,opReference);
				exit(EXIT_CODE_FAIL);
			}
			return Message_GenerateResponse(message, response, true, "", 0+1);
		} else {
			if (FileSystem_FileExists(globals.fileSystem,filename)){
				return GenerateErrorResponse(message,response, ERROR_CODE_ALREADY_STOPPED,"");
			} else {
				return GenerateErrorResponse(message,response, ERROR_CODE_NOT_SCHEDULED,"");
			}
		}
	}
}
StatusCode Command_GET(SafeMessage* message, SafeMessage* response){
	char *rdbuffer = NULL;
	char filename[17];
	size_t opReference;
	size_t mjd;
	size_t start;
	size_t length;
	ssize_t bytesRead;
	size_t fileStartPos;
	size_t chunksize=globals.fileSystem->fileSystemHeaderData->filesystemInfo.chunkSize;
	StatusCode sc;

	tempfile.index=-1;
	tempfile.isOpen=false;
	rdbuffer = (char*) mmap(NULL, chunksize, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS | MAP_POPULATE | MAP_LOCKED, -1, 0);
	strncpy(filename,message->data.messageData,17);
	filename[16]='\0';

	if (sscanf(message->data.messageData,"%6lu_%9lu %15lu %15lu",&mjd,&opReference,&start, &length)!=4){
		munmap((void*)rdbuffer,chunksize);
		return Message_GenerateResponse(message,response,false, "Bad format", strlen("Bad format")+1);
	}
	//printf("input good mjd=%lu  ref=%lu  start=%lu  len=%lu\n",mjd,opReference,start, length);
	if (length>8146){
		munmap((void*)rdbuffer,chunksize);
		return GenerateErrorResponse(message,response, ERROR_CODE_INVALID_RANGE,"");
	}
	//printf("range good\n");
	if (FileSystem_FileExists(globals.fileSystem,filename)){
		//printf("File Exists\n");
		if (FileSystem_OpenFile(globals.fileSystem,&tempfile,filename,READ)!=SUCCESS){
			munmap((void*)rdbuffer,chunksize);
			return GenerateErrorResponse(message,response, ERROR_CODE_UNKNOWN_ERROR,"");
		}
		//printf("File is opened\n");
		if (length+start-1 > globals.fileSystem->fileSystemHeaderData->fileInfo[tempfile.index].size){
			munmap((void*)rdbuffer,chunksize);
			return GenerateErrorResponse(message,response, ERROR_CODE_INVALID_RANGE,"");
		}
		//printf("File size ok\n");
		fileStartPos=globals.fileSystem->fileSystemHeaderData->fileInfo[tempfile.index].startPosition + chunksize ;
		// may require two reads if boundary overlaps chunk size
		if ( ((length+start-1) & ~(chunksize-1)) == (start & ~(chunksize-1))){
			//printf("File get in 1 read\n");
			if (_FileSystem_SynchronousRead(globals.fileSystem,(void*)rdbuffer,fileStartPos+(start & ~(chunksize-1)),chunksize,&bytesRead)!=SUCCESS || (bytesRead!=chunksize)){
				printf("File _FileSystem_SynchronousRead fail\n");
				FileSystem_CloseFile(&tempfile);
				munmap((void*)rdbuffer,chunksize);
				return GenerateErrorResponse(message,response, ERROR_CODE_UNKNOWN_ERROR,"");
			} else {
				//printf("File read #1 \n");
				memcpy((void*)packetbuf, (void*)&(rdbuffer[(start % chunksize)]),length);
			}
		} else {
			//printf("File get in 2 reads\n");
			if (_FileSystem_SynchronousRead(globals.fileSystem,(void*)rdbuffer,fileStartPos+(start & ~(chunksize-1)),chunksize,&bytesRead)!=SUCCESS || (bytesRead!=chunksize)){
				printf("File _FileSystem_SynchronousRead fail\n");
				FileSystem_CloseFile(&tempfile);
				munmap((void*)rdbuffer,chunksize);
				return GenerateErrorResponse(message,response, ERROR_CODE_UNKNOWN_ERROR,"");
			} else {
				//printf("File read #1 \n");
				memcpy((void*)packetbuf, (void*)&(rdbuffer[(start % chunksize)]),chunksize-(start % chunksize));
			}
			if (_FileSystem_SynchronousRead(globals.fileSystem,(void*)rdbuffer,fileStartPos+(start & ~(chunksize-1))+chunksize,chunksize,&bytesRead)!=SUCCESS || (bytesRead!=chunksize)){
				printf("File _FileSystem_SynchronousRead fail\n");
				FileSystem_CloseFile(&tempfile);
				munmap((void*)rdbuffer,chunksize);
				return GenerateErrorResponse(message,response, ERROR_CODE_UNKNOWN_ERROR,"");
			} else {
				//printf("File read #2 \n");
				memcpy((void*)&(packetbuf[(chunksize-(start % chunksize))]), (void*)&(rdbuffer[0]),length-(chunksize-(start % chunksize)));
			}
		}
		//printf("File read done \n");
		if(FileSystem_CloseFile(&tempfile)!=SUCCESS){
			printf("File close fail\n");
			munmap((void*)rdbuffer,chunksize);
			return GenerateErrorResponse(message,response, ERROR_CODE_UNKNOWN_ERROR,"");
		} else{
			//printf("File close success\n");
			sc = Message_GenerateResponseBinary(message,response,true,packetbuf,length);
			munmap((void*)rdbuffer,chunksize);;
			return sc;
		}
	} else {
		//printf("File not found\n");
		munmap((void*)rdbuffer,chunksize);
		return GenerateErrorResponse(message,response, ERROR_CODE_FILE_NOT_FOUND,filename);
	}
}

// Schedulable Commands (Operations)
StatusCode Command_REC(SafeMessage* message, SafeMessage* response){
	char conflictString[8192];
	OperationParameters opp,conflictOpp;
	ScheduleEntry temp;
	ScheduleEntry conflict;
	StatusCode sc;
	size_t size;
	DataFormat format;
	CompiledDataFormat compiledFormat;
	bzero((void*)&opp,sizeof(OperationParameters));
	size_t length;
	int readcount = sscanf(message->data.messageData,"%6lu %9lu %9lu %32s",&opp.opStartMJD, &opp.opStartMPM, &length, opp.opFormat);
	if (readcount!=4){
		return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_FORMAT, opp.opFormat);
	} else {
		opp.opStopMJD = opp.opStartMJD;
		opp.opStopMPM = opp.opStartMPM + length;
		if (opp.opStopMPM > MILLISECONDS_PER_DAY){
			opp.opStopMPM -= MILLISECONDS_PER_DAY;
			opp.opStopMJD++;
		}
		sprintf(opp.opDeviceName,"%s",globals.disk->deviceName);
		char buffer[10];
		strncpy(buffer,message->reference,9);
		opp.opReference=strtoul(buffer,NULL,10);
		buffer[9]='\0';
		sprintf(opp.opTag,"%06lu_%09lu",opp.opStartMJD,opp.opReference);
		opp.opType=OP_TYPE_REC;
		// now make sure we have the space
		//printf("Message: \n'%s'\n '%s'\n '%s'\n '%s'\n '%s'\n '%s'\n '%s'\n '%s'\n",message->sender, message->destination, message->type, message->reference, message->MJD, message->MPM, message->dataLength, message->data.messageData );
		if (DataFormats_Get(opp.opFormat, &format)!=SUCCESS)
			return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_FORMAT, opp.opFormat);
		if (DataFormats_Compile(&format, &compiledFormat)!=SUCCESS)
			return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_FORMAT, opp.opFormat);
		size =  (compiledFormat.finalRate * length) / 1000l;
		if (!FileSystem_IsFileCreatable(globals.fileSystem,opp.opTag,size,false)){
			if (FileSystem_FileExists(globals.fileSystem,opp.opTag)){
				return GenerateErrorResponse(message, response, ERROR_CODE_FILE_ALREADY_EXISTS, opp.opTag);
			} else {
				return GenerateErrorResponse(message, response, ERROR_CODE_INSUFFICIENT_SPACE, "");
			}
		}
		temp.reference	= opp.opReference;
		temp.startMJD   = opp.opStartMJD;
		temp.startMPM   = opp.opStartMPM;
		temp.stopMJD    = opp.opStopMJD;
		temp.stopMPM    = opp.opStopMPM;
		temp.data       = (char*)&opp;
		temp.datalength = sizeof(OperationParameters);
		if (Schedule_AddEntry(&temp,&conflict)!=SUCCESS){
			if (Schedule_EntriesEqual(&temp,&conflict)){
				return GenerateErrorResponse(message, response, ERROR_CODE_INVALID_TIME, "");
			} else {

				size_t bytes;
				sc = Schedule_GetEntryData(conflict.reference,(char*)&conflictOpp,&bytes);
				if (sc!=SUCCESS || bytes!= sizeof(OperationParameters)){
					printf("Error retrieving conflict data\nExitting\n");
					exit(EXIT_CODE_FAIL);
				}
				/*printf("%11.11s\n", Operation_TypeName(conflictOpp.opType));
				printf("%09lu\n",  conflict.reference );
				printf("%06lu\n",  conflict.startMJD);
				printf("%09lu\n", conflict.startMPM);
				printf("%06lu\n", conflict.stopMJD);
				printf("%09lu\n", conflict.stopMPM);
				printf("%32.32s\n", conflictOpp.opFormat);*/
				sprintf(conflictString,"%11.11s %09lu %06lu %09lu %06lu %09lu %32.32s", Operation_TypeName(conflictOpp.opType), conflict.reference, conflict.startMJD, conflict.startMPM, conflict.stopMJD, conflict.stopMPM, conflictOpp.opFormat);
				return GenerateErrorResponse(message, response, ERROR_CODE_TIME_CONFLICT, conflictString);
			}
		} else {
			int findex=0;
			if (FileSystem_CreateFile(globals.fileSystem,opp.opTag,size,false,&findex)!=SUCCESS){
				return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_ERROR, "");
			}
			return Message_GenerateResponse(message, response, true, opp.opTag, strlen(opp.opTag)+1);
		}
	}
}
void dumppacket(char* packet, size_t size, size_t hlStart, size_t hlStop);

StatusCode Command_TST(SafeMessage* message, SafeMessage* response){
	char *rdbuffer =  (char*) mmap(NULL, 1048576, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS | MAP_POPULATE | MAP_LOCKED, -1, 0);
	//char *rdbuffer = (char*)globals.rxQueue->data;
	//char rdbuffer[262144];

	size_t bytes_in_rdbuf=0;
	size_t bytes_in_pbuf=0;
	size_t bytesToCpy=0;
	size_t frameSize=200;
	size_t numFrames=50;
	size_t startFrame=0;
	size_t FrameCount=0;

	//char * next=NULL;
	//startFrame = strtoul(message->data.messageData, &next ,10);
	//numFrames = strtoul(next, &next ,10);
	//frameSize = strtoul(next, &next ,10);
	//strcpy(filename,"055142_000000208");
	//strncpy(filename,next+1,16);
	//filename[16]='\0';

	int count=sscanf(message->data.messageData,"%lu %lu %lu %16s", &startFrame, &numFrames, &frameSize, filename);
	//printf("data='%s'\n",message->data.messageData);
	printf("'%lu %lu %lu %s'\n", startFrame, numFrames, frameSize, filename);

	if (count!=4) return Message_GenerateResponse(message, response, false, "Bad Format", strlen("Bad Format")+1);

	tempfile.index=-1;
	tempfile.isOpen=false;
	StatusCode sc = FileSystem_OpenFile(globals.fileSystem,&tempfile,filename,READ);
	if (sc==SUCCESS){
		//printf("(first) reof %d, frm %lu, bytes_in_pbuf %lu, bytes_in_rdbuf %lu\n",myfile.reof, numFrames,bytes_in_pbuf, bytes_in_rdbuf);
		while (!tempfile.reof && numFrames){
			//printf("reof %d, frm %lu, bytes_in_pbuf %lu, bytes_in_rdbuf %lu\n",myfile.reof, numFrames,bytes_in_pbuf, bytes_in_rdbuf);
			if (bytes_in_pbuf && bytes_in_rdbuf) {
				bytesToCpy=(bytes_in_rdbuf<(frameSize-bytes_in_pbuf)) ? bytes_in_rdbuf : (frameSize-bytes_in_pbuf);
				memcpy( (void *) (&(packetbuf[bytes_in_pbuf])), (void *) &rdbuffer[262144-bytes_in_rdbuf], bytesToCpy);
				bytes_in_pbuf+=bytesToCpy;
				if (bytes_in_pbuf>=frameSize){
					if (++FrameCount>startFrame) dumppacket(packetbuf,frameSize,0,0);
					numFrames--;
					bytes_in_pbuf-=frameSize;
					if (bytes_in_pbuf!=0){printf("Algorithm error\n"); exit(EXIT_CODE_FAIL);}
				}
				bytes_in_rdbuf-=bytesToCpy;
			} else if (bytes_in_rdbuf) {
				bytesToCpy=(bytes_in_rdbuf<frameSize) ? bytes_in_rdbuf : frameSize;
				memcpy( (void *) (&(packetbuf[bytes_in_pbuf])), (void *) &rdbuffer[262144-bytes_in_rdbuf], bytesToCpy);
				bytes_in_pbuf+=bytesToCpy;
				if (bytes_in_pbuf>=frameSize){
					if (++FrameCount>startFrame) dumppacket(packetbuf,frameSize,0,0);
					numFrames--;
					bytes_in_pbuf-=frameSize;
					if (bytes_in_pbuf!=0){printf("Algorithm error\n"); exit(EXIT_CODE_FAIL);}
				}
				bytes_in_rdbuf-=bytesToCpy;
			} else {
				sc=FileSystem_AsynchronousReadStart(&tempfile,rdbuffer,262144);
				if (sc!=SUCCESS){printf("File Read Failure\n"); exit(EXIT_CODE_FAIL);}
				while ((sc=FileSystem_IsAsynchronousReadDone(&tempfile))==NOT_READY){
					usleep(10000);//wait
				}
				if (sc!=SUCCESS){printf("File Read Failure\n"); exit(EXIT_CODE_FAIL);}
				if(FileSystem_GetAsynchronousBytesRead(&tempfile,&bytes_in_rdbuf)!=SUCCESS){printf("File Read Failure\n"); exit(EXIT_CODE_FAIL);}
			}
		}
		if(FileSystem_CloseFile(&tempfile)!=SUCCESS){printf("File Close Failure\n"); exit(EXIT_CODE_FAIL);}
		printf("Finish dump\n"); fflush(stdout);
	} else {
		munmap((void*)rdbuffer , 1048576);
		return Message_GenerateResponse(message, response, false, "File Open Failure",strlen("File Open Failure") );
	}
	munmap((void*)rdbuffer , 1048576);
	return Message_GenerateResponse(message, response, false, "abcd", 0);

}

// Non-instant, Non-schedulable Commands (Operations)
StatusCode Command_SYN(SafeMessage* message, SafeMessage* response){
	StatusCode sc;
	char command[8192];
	sprintf(command, "ntpdate %s", globals.ntpServer);
	char result[8192];
	sc= ShellGetString(command, result, 8192);
	if (sc!=SUCCESS){
		printf("Error: NTP synchronization process failed\n");
		return GenerateErrorResponse(message,response, ERROR_CODE_UNKNOWN_ERROR,"");
	} else {
		return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
	}
}

StatusCode Command_CPY(SafeMessage* message, SafeMessage* response){
	OperationParameters opp;
	ScheduleEntry temp;
	ScheduleEntry conflict;
	int diskindex;
	TimeStamp now;
	int fileIndex;
	bzero((void*)&opp,sizeof(OperationParameters));


	char * token;
	token = strtok(message->data.messageData," ");  if (!token) return Message_GenerateResponse(message,response,false, "Bad format", strlen("Bad format")+1);
	strncpy(opp.opTag,token,16);	opp.opTag[16]='\0';

	token = strtok(NULL," "); 						if (!token) return Message_GenerateResponse(message,response,false, "Bad format", strlen("Bad format")+1);
	opp.opFileCpyDmpStartPos = strtoul(token,NULL,0);

	token = strtok(NULL," "); 						if (!token) return Message_GenerateResponse(message,response,false, "Bad format", strlen("Bad format")+1);
	opp.opFileCpyDmpLength = strtoul(token,NULL,0);

	token = strtok(message->data.messageData," ");  if (!token) return Message_GenerateResponse(message,response,false, "Bad format", strlen("Bad format")+1);
	strncpy(opp.opDeviceName,token,64);		opp.opDeviceName[63]='\0';

	token = strtok(message->data.messageData," ");  if (!token) return Message_GenerateResponse(message,response,false, "Bad format", strlen("Bad format")+1);
	strncpy(opp.opFileName,token,128);		opp.opFileName[127]='\0';


	fileIndex = _FileSystem_GetFileIndex(globals.fileSystem,opp.opTag);
	if (fileIndex != -1){
		if ((opp.opFileCpyDmpStartPos + opp.opFileCpyDmpLength-1) > globals.fileSystem->fileSystemHeaderData->fileInfo[fileIndex].size){
			return GenerateErrorResponse(message,response, ERROR_CODE_INVALID_RANGE,"");
		}
		if (strspn(opp.opFileName,"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_.")!=strlen(opp.opFileName)){
			return GenerateErrorResponse(message,response, ERROR_CODE_INVALID_FILENAME,opp.opFileName);
		}
		if (strstr(opp.opFileName,"..")!=NULL){
			return GenerateErrorResponse(message,response, ERROR_CODE_INVALID_FILENAME,opp.opFileName);
		}
		now = getTimestamp();
		addTime(now.MJD, now.MPM, 3000, &opp.opStartMJD, &opp.opStartMPM);
		opp.opStopMJD = 999999l;
		opp.opStopMPM = 999999999l;
		diskindex = DiskFindIndexByName(opp.opDeviceName);
		if (diskindex == -1){
			return GenerateErrorResponse(message, response, ERROR_CODE_INVALID_STORAGE_ID, opp.opDeviceName);
		} else {
			if (DiskGetType(diskindex)!=DRIVETYPE_EXTERNAL_PARTITION){
				return GenerateErrorResponse(message, response, ERROR_CODE_INVALID_STORAGE_ID, opp.opDeviceName);
			} else {
				if (DiskGetFreeSpace(diskindex)<(opp.opFileCpyDmpLength*11/10)){
					return GenerateErrorResponse(message, response, ERROR_CODE_INSUFFICIENT_SPACE,"");
				} else {
					opp.opFileDmpBlockSize=opp.opFileCpyDmpLength;
					strcpy(opp.opFormat,"<<<    not implemented yet   >>>");
					opp.opReference=strtoul(message->reference,NULL,10);
					opp.opType=OP_TYPE_CPY;
					temp.reference	= opp.opReference;
					temp.startMJD   = opp.opStartMJD;
					temp.startMPM   = opp.opStartMPM;
					temp.stopMJD    = opp.opStopMJD;
					temp.stopMPM    = opp.opStopMPM;
					temp.data       = (char*)&opp;
					temp.datalength = sizeof(OperationParameters);
					if (Schedule_AddEntry(&temp,&conflict)!=SUCCESS){
						if (Schedule_EntriesEqual(&temp,&conflict)){
							return GenerateErrorResponse(message, response, ERROR_CODE_OPERATION_NOT_PERMITTED, "");
						}
					} else {
						return Message_GenerateResponse(message, response, true, "", 0);
					}
				}
			}
		}
	} else {
		return GenerateErrorResponse(message,response, ERROR_CODE_FILE_NOT_FOUND,opp.opTag);
	}
}

StatusCode Command_DMP(SafeMessage* message, SafeMessage* response){
	return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
}
StatusCode Command_FMT(SafeMessage* message, SafeMessage* response){
	return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
}
StatusCode Command_DWN(SafeMessage* message, SafeMessage* response){
	return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
}
StatusCode Command_UP (SafeMessage* message, SafeMessage* response){
	return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
}
StatusCode Command_EJT(SafeMessage* message, SafeMessage* response){
	return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
}







//ANSI color console text ===================================================================================================
#define COLOR_NORMAL "\033[1;37m"
#define COLOR_RED "\033[0;31m"
#define COLOR_GREEN "\033[0;32m"
#define COLOR_GREY "\033[1;30m"
#define HIGHLIGHT_GOOD(a)(COLOR_GREEN a COLOR_NORMAL)
#define HIGHLIGHT_BAD(a)(COLOR_RED a COLOR_NORMAL)

void dumppacket(char* packet, size_t size, size_t hlStart, size_t hlStop){
	static size_t lastsid=0;
	static size_t sid=0;
	sid=((size_t*)packet)[1];
	printf(COLOR_NORMAL);
	printf(" __________________________________________________________________________________\n");
	printf(">MPM   = %lu\n",*((size_t*)packet));
	if (sid!=(lastsid+1))
		printf(HIGHLIGHT_BAD(">frame = %lu\n"),((size_t*)packet)[1]);
	else
		printf(HIGHLIGHT_GOOD(">frame = %lu\n"),((size_t*)packet)[1]);
	lastsid=sid;
	printf("              ____________________________________________________________________ \n");
	printf("              |  00  01  02  03  04  05  06  07  08  09  0A  0B  0C  0D  0E  0F   |\n");
	printf(" _____________|___________________________________________________________________|\n");
	size_t row=0;
	size_t col=0;
	for(row=0;row<((size/16)+1);row++){
		printf(COLOR_NORMAL);
	    printf("|%12lX |  ",row);
		for (col=row*16; ((col<((row*16)+16)) ) ;col++){
			if (col<hlStart) printf(COLOR_GREEN);
			if (col>=hlStart) printf(COLOR_RED);
			if (col>=hlStop) printf(COLOR_GREY);
			if (col<size)
				printf("%.2x  ",((char)packet[col])&0xff);
			else
				printf("    ");

		}
		printf(COLOR_NORMAL);
		printf(" | ");
		for (col=row*16; ((col<((row*16)+16)) ) ;col++){
			if (col<size && isprint(packet[col]))
				printf("%c",packet[col]);
			else
				printf(".");
		}
		printf("\n");
	}
	printf(COLOR_NORMAL);
	printf("|_____________|___________________________________________________________________|\n");
}

