/*
 * MIB.c
 *
 *  Created on: Oct 29, 2009
 *      Author: chwolfe2
 */
#include "MIB.h"
#include <stdlib.h>
#include <string.h>

#include "Globals.h"

StatusCode _MIB_Get_Operation(SafeMessage* message, SafeMessage* response){
	char buffer[8192];
	char result[MESSAGE_FIELDSIZE_DATA];
	strncpy(buffer,message->data.messageData, MESSAGE_FIELDSIZE_DATA);
	buffer[MESSAGE_FIELDSIZE_DATA]='\0';
	if 	 	  (strncmp(buffer, "OP-TYPE", 		   7) == 0) {
		sprintf(result,"%s",Operation_TypeName(globals.opData.opType));
		return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
	} else if (strncmp(buffer, "OP-START", 		   8) == 0) {
		sprintf(result,"%06lu %09lu",globals.opData.opStartMJD,globals.opData.opStartMPM);
		return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
	} else if (strncmp(buffer, "OP-STOP", 	  	   7) == 0) {
		sprintf(result,"%06lu %09lu",globals.opData.opStopMJD,globals.opData.opStopMPM);
		return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
	} else if (strncmp(buffer, "OP-REFERENCE",    12) == 0) {
		sprintf(result,"%09lu",globals.opData.opReference);
		return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
	} else if (strncmp(buffer, "OP-ERRORS", 	   9) == 0) {
		sprintf(result,"%015lu %015lu",globals.opData.errorCount,globals.opData.warningCount);
		return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
	} else if (strncmp(buffer, "OP-TAG", 		   6) == 0) {
		snprintf(result,17,"%s",globals.opData.opTag);
		return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
	} else if (strncmp(buffer, "OP-FORMAT", 	   9) == 0) {
		snprintf(result,DATA_FORMAT_MAX_NAME_LENGTH+1,"%s",globals.opData.opFormat);
		return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
	} else if (strncmp(buffer, "OP-FILEPOSITION", 15) == 0) {
		sprintf(result,"%015lu %015lu %015lu",globals.opData.opFileCpyDmpStartPos,globals.opData.opFileCpyDmpLength,globals.opData.opFileCpyDmpCurrentPos);
		return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
	} else if (strncmp(buffer, "OP-FILENAME", 	  11) == 0) {
		sprintf(result,"%-*s %-*s",MAX_DEVICE_NAME_LENGTH,globals.opData.extdisk->deviceName,EXT_FILE_MAX_NAME_LENGTH,globals.opData.opFileName);
		return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
	} else if (strncmp(buffer, "OP-FILEINDEX", 	  12) == 0) {
		sprintf(result,"%09lu",globals.opData.opFileDmpCurrentIndex);
		return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
	} else return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
	return FAILURE;
}

StatusCode _MIB_Get_Schedule(SafeMessage* message, SafeMessage* response){
	char buffer[8192];
	strncpy(buffer,message->data.messageData, MESSAGE_FIELDSIZE_DATA);
	buffer[MESSAGE_FIELDSIZE_DATA]='\0';
	//char result[MESSAGE_FIELDSIZE_DATA];
	if 	 	  (strncmp(buffer, "SCHEDULE-COUNT",   14) == 0) {
		int x= _Schedule_Count();
		if (x==-1){
			return Message_GenerateResponse(message, response, true, "000000", strlen("000000")+1);
		} else {
			char result[40];
			sprintf(result,"%06d",x);
			return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
		}
	} else if (strncmp(buffer, "SCHEDULE-ENTRY-",  15) == 0) {
		OperationParameters opp;
		ScheduleEntry entry;
		char result[40];
		strncpy(result,&buffer[15],6); result[6]='\0';
		int index=atoi(result);
		size_t bytes=0;
		if (Schedule_GetEntryIndex(index,&entry)!=SUCCESS) return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
		if (Schedule_GetEntryDataIndex(index,(char*)&opp, &bytes)!=SUCCESS) return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
		char opp_string[8192];
		sprintf(opp_string,"%11.11s %09lu %06lu %09lu %06lu %09lu %32.32s", Operation_TypeName(opp.opType), entry.reference, entry.startMJD, entry.startMPM, entry.stopMJD, entry.stopMPM, opp.opFormat);
		return Message_GenerateResponse(message, response, true, opp_string, strlen(opp_string)+1);
	} else return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
	return FAILURE;
}

StatusCode _MIB_Get_Directory(SafeMessage* message, SafeMessage* response){
	char buffer[8192];
	strncpy(buffer,message->data.messageData, MESSAGE_FIELDSIZE_DATA);
	buffer[MESSAGE_FIELDSIZE_DATA]='\0';
	char result[MESSAGE_FIELDSIZE_DATA];
	if 	 	  (strncmp(buffer, "DIRECTORY-COUNT",   15) == 0) {
		size_t count;
		count = FileSystem_GetFileCount(globals.fileSystem);
		sprintf(result,"%06lu",count);
		return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
	} else if (strncmp(buffer, "DIRECTORY-ENTRY-",  16) == 0) {
		strncpy(result, &message->data.messageData[16], 6);
		result[6]='\0';
		size_t index=strtoul(result,NULL,10);
		size_t count;
		count = FileSystem_GetFileCount(globals.fileSystem);
		if (index<count){
			FileSystem_GetFileMetaData(globals.fileSystem,index,(FileMetaData*) result);
			return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
		} else {
			return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
		}
	} else return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
	return FAILURE;
}

StatusCode _MIB_Get_Devices(SafeMessage* message, SafeMessage* response){
	char buffer[8192];
	strncpy(buffer,message->data.messageData, MESSAGE_FIELDSIZE_DATA);
	buffer[MESSAGE_FIELDSIZE_DATA]='\0';
	if 	 	  (strncmp(buffer, "DEVICE-COUNT",      12) == 0) {
		return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
	} else if (strncmp(buffer, "DEVICE-ID-",        10) == 0) {
		return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
	} else if (strncmp(buffer, "DEVICE-STORAGE-",   15) == 0) {
		return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
	} else if (strncmp(buffer, "TOTAL-STORAGE",     13) == 0) {
		return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
	} else if (strncmp(buffer, "REMAINING-STORAGE", 17) == 0) {
		return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
	} else return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
	return FAILURE;
}

StatusCode _MIB_Get_CPU(SafeMessage* message, SafeMessage* response){
	char buffer[8192];
	strncpy(buffer,message->data.messageData, MESSAGE_FIELDSIZE_DATA);
	buffer[MESSAGE_FIELDSIZE_DATA]='\0';
	if 	 	  (strncmp(buffer, "CPU-COUNT",      9) == 0) {
		return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
	} else if (strncmp(buffer, "CPU-TEMP-",      9) == 0) {
		return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
	} else return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
	return FAILURE;
}

StatusCode _MIB_Get_HDD(SafeMessage* message, SafeMessage* response){
	char buffer[8192];
	strncpy(buffer,message->data.messageData, MESSAGE_FIELDSIZE_DATA);
	buffer[MESSAGE_FIELDSIZE_DATA]='\0';
	if 	 	  (strncmp(buffer, "HDD-COUNT",      9) == 0) {
		return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
	} else if (strncmp(buffer, "HDD-TEMP-",      9) == 0) {
		return Message_GenerateResponse(message, response, false, "NOT IMPLEMENTED YET", strlen("NOT IMPLEMENTED YET")+1);
	} else return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
	return FAILURE;
}

/*
StatusCode DataFormats_GetCount			(int* count);
StatusCode DataFormats_GetName			(int index, char * formatName);
StatusCode DataFormats_GetSpec			(int index, char * spec);
StatusCode DataFormats_GetPayloadSize	(int index, size_t * payloadSize);
StatusCode DataFormats_GetRate			(int index, size_t * rate);
*/

StatusCode _MIB_Get_Formats(SafeMessage* message, SafeMessage* response){
	char buffer[8192];
	strncpy(buffer,message->data.messageData, MESSAGE_FIELDSIZE_DATA);
	buffer[MESSAGE_FIELDSIZE_DATA]='\0';
	char result[MESSAGE_FIELDSIZE_DATA];
	if 	 	  (strncmp(buffer, "FORMAT-COUNT",    12) == 0) {
		int count;
		StatusCode sc = DataFormats_GetCount(&count);
		if (sc==SUCCESS){
			sprintf(result,"%06d", count);
			return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
		} else {
			return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_ERROR, "");
		}
	} else if (strncmp(buffer, "FORMAT-NAME-",    12) == 0) {
		strncpy(result, &message->data.messageData[12], 6);
		result[6]='\0';
		int index=atoi(result);
		StatusCode sc = DataFormats_GetName(index,result);
		if (sc==SUCCESS){
			return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
		} else {
			return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
		}
	} else if (strncmp(buffer, "FORMAT-PAYLOAD-", 15) == 0) {
		strncpy(result, &message->data.messageData[15], 6);
		result[6]='\0';
		int index=atoi(result);
		size_t PayloadSize;
		StatusCode sc = DataFormats_GetPayloadSize(index,&PayloadSize);
		if (sc==SUCCESS){
			sprintf(result,"%04lu",PayloadSize);
			return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
		} else {
			return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
		}
	} else if (strncmp(buffer, "FORMAT-RATE-",    12) == 0) {
		strncpy(result, &message->data.messageData[15], 6);
		result[6]='\0';
		int index=atoi(result);
		size_t rate;
		StatusCode sc = DataFormats_GetRate(index,&rate);
		if (sc==SUCCESS){
			sprintf(result,"%015lu",rate);
			return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
		} else {
			return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
		}
	} else if (strncmp(buffer, "FORMAT-SPEC-",    12) == 0) {
		strncpy(result, &message->data.messageData[12], 6);
		result[6]='\0';
		int index=atoi(result);
		StatusCode sc = DataFormats_GetSpec(index,result);
		if (sc==SUCCESS){
			return Message_GenerateResponse(message, response, true, result, strlen(result)+1);
		} else {
			return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
		}
	} else return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
	return FAILURE;
}

StatusCode _MIB_Get_Log(SafeMessage* message, SafeMessage* response){
	char buffer[8192];
	strncpy(buffer,message->data.messageData, MESSAGE_FIELDSIZE_DATA);
	buffer[MESSAGE_FIELDSIZE_DATA]='\0';
	if 	 	  (strncmp(buffer, "LOG-COUNT",      9) == 0) {
	} else if (strncmp(buffer, "LOG-ENTRY-",    10) == 0) {
	} else return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
	return FAILURE;
}


StatusCode MIB_Get(SafeMessage* message, SafeMessage* response){
	if ((message==NULL) || (response==NULL) ) return FAILURE;
	char buffer[8192];
	strncpy(buffer,message->data.messageData, MESSAGE_FIELDSIZE_DATA);
	buffer[MESSAGE_FIELDSIZE_DATA]='\0';
	if 	 	  (strncmp(buffer, "OP", 2) == 0)   return _MIB_Get_Operation(message,response);
	else   if (strncmp(buffer, "SC", 2) == 0)   return _MIB_Get_Schedule(message,response);
	else   if (strncmp(buffer, "DI", 2) == 0)   return _MIB_Get_Directory(message,response);
	else   if (strncmp(buffer, "CP", 2) == 0)   return _MIB_Get_CPU(message,response);
	else   if (strncmp(buffer, "HD", 2) == 0)   return _MIB_Get_HDD(message,response);
	else   if (strncmp(buffer, "FO", 2) == 0)   return _MIB_Get_Formats(message,response);
	else   if (strncmp(buffer, "LO", 2) == 0)   return _MIB_Get_Log(message,response);
	else   if (strncmp(buffer, "DE", 2) == 0)   return _MIB_Get_Devices(message,response);
	else   if (strncmp(buffer, "TO", 2) == 0)   return _MIB_Get_Devices(message,response);
	else   if (strncmp(buffer, "RE", 2) == 0)   return _MIB_Get_Devices(message,response);
	else   return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, buffer);
	return FAILURE;
}
/*
if (sc==SUCCESS){
		return Message_GenerateResponse(message, response, true, buffer, strlen(buffer));
	} else {
		return GenerateErrorResponse(message, response, ERROR_CODE_UNKNOWN_MIB_ENTRY, request);
	}
*/
