/*
 * Operations.c
 *
 *  Created on: Oct 31, 2009
 *      Author: chwolfe2
 */

#include "Operations.h"
#include "Globals.h"
#include <assert.h>
#include <string.h>


void process_REC();
void receiveEnqueue();
void dequeueWrite();

char * TYPE_NAMES[]={
		"Idle       ",
		"Record     ",
		"Test       ",
		"Synchronize",
		"Format     ",
		"Copy       ",
		"Dump       ",
		"Down       "
};




boolean	formatApplied 		= false;
boolean	dataReceived 		= false;
boolean	insertionAvailable 	= false;
boolean writeComplete 		= false;
boolean	removalAvailable 	= false;
boolean	writeInProgress 	= false;





char * Operation_TypeName(OpType type){
	return TYPE_NAMES[type];
}

// fill out the Operation Data struct from saved (in the schedule) params.
StatusCode Operation_Init(OperationParameters* params){
	globals.opData.initialParameters=params;
	globals.opData.errorCount=0;
	globals.opData.warningCount=0;
	globals.opData.opFileCpyDmpCurrentPos=0;
	globals.opData.opFileCpyDmpLength=0;
	globals.opData.opFileDmpBlockSize=0;
	globals.opData.opStartMJD = params->opStartMJD;
	globals.opData.opStartMPM = params->opStartMPM;
	globals.opData.opStopMJD = params->opStopMJD;
	globals.opData.opStopMPM = params->opStopMPM;
	globals.opData.opReference = params->opReference;
	globals.opData.extFileHandle = -1;
	globals.opData.opFileCpyDmpStartPos = 0;
	globals.opData.opFileDmpCurrentIndex = 0;
	globals.opData.opStatus = OP_STATUS_NULL;
	globals.opData.opType = params->opType;
	strncpy(globals.opData.opDeviceName,params->opDeviceName,MAX_DEVICE_NAME_LENGTH+1);
	strncpy(globals.opData.opFormat,params->opFormat,DATA_FORMAT_MAX_NAME_LENGTH+1);
	strncpy(globals.opData.opFileName,params->opFileName,EXT_FILE_MAX_NAME_LENGTH+1);
	strncpy(globals.opData.opTag,params->opTag,16+1);

/*	params->opDeviceName;
	params->opFileCpyDmpLength;
	params->opFileCpyDmpStartPos;
	params->opFileDmpBlockSize;
	params->opFileName;
	params->opReference;
	params->opStartMJD;
	params->opStartMPM;
	params->opStopMJD;
	params->opStopMPM;
	params->opTag;
*/
	switch(params->opType){
		case OP_TYPE_IDLE		:
			return SUCCESS;
			break;
		case OP_TYPE_REC		:
			formatApplied 		= false;
			dataReceived 		= false;
			insertionAvailable 	= false;
			writeComplete 		= false;
			removalAvailable 	= false;
			writeInProgress 	= false;
			globals.opData.rxSocketOpen = false;
			globals.opData.tagfile->index=-1;
			if (DataFormats_Get(params->opFormat,globals.opData.dataFormat)==SUCCESS){
				if (DataFormats_Compile(globals.opData.dataFormat, globals.opData.compiledFormat)==SUCCESS){
					if (FileSystem_OpenFile(globals.fileSystem,globals.opData.tagfile,params->opTag,WRITE)==SUCCESS){
						if(RingQueue_Reset(globals.rxQueue)==SUCCESS){
							globals.opData.opStatus =OP_STATUS_INITIALIZED;
							return SUCCESS;
						} else {// reset ring queue
							printf("Error: RingQueue could not be flushed.\n");
						}
					} else {// open tag file
						printf("Error: Tag file could not be opened.\n");
					}
				} else {// compile format
					printf("Error: Data format could not be compiled.\n");
				}
			} else {// get recording format
				printf("Error: Data format could not be identified.\n");
			}
			break;
		case OP_TYPE_TST		:
			break;
		case OP_TYPE_FMT		:
			break;
		case OP_TYPE_CPY		:
			break;
		case OP_TYPE_DMP		:
			break;
		case OP_TYPE_DWN		:
			break;
		default :
			printf("Unsupported operation type.\n");
		break;
	}

	return FAILURE;
}

// Do the work of the operation. returns NOT_READY until complete, and then SUCCESS. FAILURE means FAILURE.
StatusCode Operation_Run(){
	switch(globals.opData.opType){
		case OP_TYPE_IDLE		: return SUCCESS; break;
		case OP_TYPE_REC		:
			if (!globals.opData.rxSocketOpen){
				printf("Opening socket on port %du\n",globals.DataInPort);
				if (Socket_OpenRecieve(globals.rxSocket,globals.DataInPort)!=SUCCESS){
					printf("Error: couldn't open data socket.\n");
					exit(EXIT_CODE_FAIL);
				}
				globals.opData.opStatus =OP_STATUS_RUNNING;
				globals.opData.rxSocketOpen = true;
			}
			// this is where the data path is handled....
			process_REC();
			return SUCCESS;
			break;
		case OP_TYPE_TST		: return SUCCESS; break;
		case OP_TYPE_FMT		: return SUCCESS; break;
		case OP_TYPE_CPY		: return SUCCESS; break;
		case OP_TYPE_DMP		: return SUCCESS; break;
		case OP_TYPE_DWN		: return SUCCESS; break;
		default :
			printf("Unsupported operation type.\n");
		break;
	}
	return FAILURE;
}

// Do the cleanup required of the operation (close files, etc). when complete data will be in idle state
StatusCode Operation_CleanUp(){
	switch(globals.opData.opType){
		case OP_TYPE_IDLE		: return SUCCESS; break;
		case OP_TYPE_REC		:
			if (globals.opData.rxSocketOpen){
				if (Socket_Close(globals.rxSocket)!=SUCCESS){
					printf("Error: couldn't close data socket.\n");
					exit(EXIT_CODE_FAIL);
				}
				globals.opData.rxSocketOpen = false;
			}
			if (	globals.opData.opStatus>=OP_STATUS_RUNNING){
				globals.rxQueue->allowUndersizeRemoval=true;
				while (globals.rxQueue->bytesUsed){
					dequeueWrite();
				}
			}
			if (globals.opData.tagfile->isOpen){
				if (FileSystem_CloseFile(globals.opData.tagfile)!=SUCCESS){
					printf("Error: couldn't close tag file.\n");
					exit(EXIT_CODE_FAIL);
				}
			}
			globals.opData.opStatus =OP_STATUS_COMPLETE;
			return SUCCESS;
			break;
		case OP_TYPE_TST		: return SUCCESS; break;
		case OP_TYPE_FMT		: return SUCCESS; break;
		case OP_TYPE_CPY		: return SUCCESS; break;
		case OP_TYPE_DMP		: return SUCCESS; break;
		case OP_TYPE_DWN		: return SUCCESS; break;
		default :
			printf("Unsupported operation type.\n");
		break;
	}
	return FAILURE;
}
/*
boolean Operation_ParamsEqual(OperationParameters* a,OperationParameters* b){
	assert(a!=NULL && b!=NULL);
	if (a==b) return true;
	char * c=(char*)a;
	char * d=(char*)b;
	int i;
	for (i=0;i<sizeof(OperationParameters);i++){
		if (c[i]!=d[i]) return false;
	}
	return true;
}

*/



size_t bytesReceived;
char * inPtr=NULL;      // pointer to message waiting to be enqueued
size_t bytesWritten;
char * outPtr=NULL;      // pointer to queued data waiting to be written

void receiveEnqueue(){
	StatusCode sc;
	static size_t rptcntr=0;
	if (++rptcntr>300000l){
		printf("Bytes Recieved = %015lu\n",globals.rxSocket->bytes);
		rptcntr = 0;
	}

// ----------------------------------------- receive into data buffer
	if (!insertionAvailable){
		if(RingQueue_NextInsertionPoint(globals.rxQueue,&inPtr)==SUCCESS){
			assert (inPtr!=NULL);
			insertionAvailable=true;
		}
	}
	if (insertionAvailable && !dataReceived){
		if ((sc=Socket_Recieve(globals.rxSocket,inPtr,&bytesReceived))==SUCCESS){
			dataReceived=true;
			//printf("received %lu\n",bytesReceived);
			if (bytesReceived!=globals.opData.compiledFormat->rawPacketSize){
				globals.opData.warningCount++;
			}
		} else {
			//printf("recv says %s\n", (sc==NOT_READY ? "NOT_READY" : "FAILURE"));
		}
	}
	if(dataReceived && !formatApplied){
		if (DataFormats_Apply(globals.opData.compiledFormat, inPtr)==SUCCESS){
			//printf("format applied %lu\n",globals.opData.compiledFormat->finalPacketSize);
			formatApplied=true;
		} else {
			printf("Error: data formatting failed.\n");
			exit(EXIT_CODE_FAIL);
		}
	}
	if(formatApplied){
		if (RingQueue_NotifyInsertionComplete(globals.rxQueue,globals.opData.compiledFormat->finalPacketSize)!=SUCCESS){
			printf("Error: data buffering failed.\n");
			exit(EXIT_CODE_FAIL);
		} else {
			//printf("recv cycle complete, %lu bytes total in queue\n",globals.rxQueue->bytesUsed);
			formatApplied 		= false;
			dataReceived 		= false;
			insertionAvailable 	= false;
		}
	}
}

void dequeueWrite(){
// ----------------------------------------- write from buffer into file
	if (!writeInProgress){
		if (!removalAvailable){
			if (RingQueue_NextRemovalPoint(globals.rxQueue,&outPtr)==SUCCESS){
			//	printf("Removing...\n");
				assert (outPtr!=NULL);
				removalAvailable=true;
			}
		}
		if(removalAvailable){
			if (FileSystem_AsynchronousWriteStart(globals.opData.tagfile,outPtr,globals.fileSystem->fileSystemHeaderData->filesystemInfo.chunkSize)==SUCCESS){
			//	printf("write started ...\n");
				writeInProgress=true;
			}
		}
	}
	if (writeInProgress){
		if (! writeComplete){
			if (FileSystem_IsAsynchronousWriteDone(globals.opData.tagfile)!=NOT_READY){
				//printf("write complete ...\n");
				if (FileSystem_GetAsynchronousBytesWritten(globals.opData.tagfile,&bytesWritten)!=SUCCESS || bytesWritten!=globals.fileSystem->fileSystemHeaderData->filesystemInfo.chunkSize){
					globals.opData.errorCount++;
				}
				writeComplete=true;
			}
		}
		if (writeComplete){
			if (RingQueue_NotifyRemovalComplete(globals.rxQueue, bytesWritten)!=SUCCESS){

				printf("Error: data buffering failed.\n");
				exit(EXIT_CODE_FAIL);
			}
			//printf("removal complete ...\n");
			writeComplete = false;
			removalAvailable = false;
			writeInProgress = false;
		}

	}
}
void process_REC(){
	receiveEnqueue();
	dequeueWrite();
}



