/*
 * RingQueue.h
 *
 *  Created on: Oct 18, 2009
 *      Author: chwolfe2
 */

#ifndef RINGQUEUE_H_
#define RINGQUEUE_H_
#include "Defines.h"
#include <stdlib.h>

// size to allocate for ring queue. is 1.5 GiB
#define RING_QUEUE_STATIC_SIZE 1610612736

// size of removal requests. 256kB corresponds to size of packet written to hard drive.
#define RING_QUEUE_REMOVAL_SIZE 262144


typedef struct __ringqueue{
  void * data;
  size_t in;
  size_t out;
  size_t totalSize;
  size_t bytesUsed;
  size_t allocatedSize;
  size_t insertionSize;
  boolean allowUndersizeRemoval;
} RingQueue;

StatusCode RingQueue_Create(RingQueue* rq, size_t insertionSize);
StatusCode RingQueue_Destroy(RingQueue* rq);
StatusCode RingQueue_Reset(RingQueue* rq);
StatusCode RingQueue_NextInsertionPoint(RingQueue* rq, char** insertionPoint);
StatusCode RingQueue_NotifyInsertionComplete(RingQueue* rq, size_t bytesInserted);
StatusCode RingQueue_NextRemovalPoint(RingQueue* rq, char** removalPoint);
StatusCode RingQueue_NotifyRemovalComplete(RingQueue* rq, size_t bytesRemoved);

#endif /* RINGQUEUE_H_ */
